#!/bin/sh

if [ "$(id -u)" -ne 0 ]; then
  echo "Error: This script must be run by superuser"
  exit 1
fi

export LC_ALL=C

showprogress() {
  progress=$((progress+1))
  if [ ${count} -gt 0 -a -n "${COM_DRWEB_PROGRESS_FD}" ]; then
    echo "$((progress*100/count))" >&${COM_DRWEB_PROGRESS_FD}
  fi
}

count=$(grep -c '^showprogress$' $0)
progress=0

realpath() {
  local p=$1
  if [ "${p%${p#?}}" != "/" ]; then
    p="$(pwd)/$p"
  fi
  echo "$p"
}

n="drweb.report.$(date +'%Y%m%d%H%M%S').$$"
base="/root"
output=""
if [ $# -ge 1 ]; then
  output=$(realpath "$1")
  base=$(dirname "${output}")
fi
if [ ! -d "${base}" ]; then
  echo "Error: directory ${base} does not exists."
  exit 1
fi
if [ -n "${output}" ]; then
  if [ -e "${output}" ]; then
    echo "Error: path ${output} already exists."
    exit 1
  fi
fi
d="${base}/$n"

echo "The script is gathering required information. This may take some time, please wait..."

trap "rm -rf '$d'" EXIT HUP INT PIPE TERM
mkdir -p "$d"
cd "$d"

#ОС
uname -o > OS 2>&1
showprogress

#архитектуру
uname -m > arch 2>&1
showprogress

#вывод uname -a
uname -a > uname 2>&1
showprogress

#получение информации о версии дистрибутива
lsb_release -a > release 2>&1
showprogress
freebsd-version >> release 2>&1
showprogress
head /etc/*release* /etc/*issue* >> release 2>&1
showprogress

#список пакетов - (rpm -qa, dpkg -l, pkg (freebsd),)
if type rpm >/dev/null 2>&1; then
  rpm -qa > rpm.txt 2>&1
fi
showprogress
if type dpkg >/dev/null 2>&1; then
  dpkg -l > dpkg.txt 2>&1
fi
showprogress
if type pkg >/dev/null 2>&1; then
  pkg info < /dev/null > pkg.txt 2>&1
fi
showprogress

#список наших пакетов - .../drweb.com/bin/rpm -qa
/opt/drweb.com/bin/rpm -qa > drweb.rpm.txt 2>&1
showprogress

mkdir -p logs
#лог syslog (/var/log/syslog, /var/log/messages)
cp /var/log/syslog logs/ >> logs/copy.txt 2>&1
showprogress
cp /var/log/messages logs/ >> logs/copy.txt 2>&1
showprogress
cp -r /var/opt/drweb.com/log/* logs/ >> logs/copy.txt 2>&1
showprogress

#лог пакетного менеджера (/var/log/apt/, /var/log/yum, etc...)
cp /var/log/apt/history.log /var/log/apt/term.log logs/ >> logs/copy.txt 2>&1
showprogress
cp /var/log/yum.log logs/ >> logs/copy.txt 2>&1
showprogress
cp /var/log/apt/dnf*log logs/ >> logs/copy.txt 2>&1
showprogress
cp /var/log/zypper.log logs/ >> logs/copy.txt 2>&1
showprogress

#список и md5 файлов в .../drweb.com/ (/opt и /var)
if type md5sum >/dev/null 2>&1; then
  md5sum=md5sum
elif type md5 >/dev/null 2>&1; then
  md5sum='md5 -r'
elif type openssl >/dev/null 2>&1; then
  md5sum='openssl md5 -r'
fi
if [ -n "$md5sum" ]; then
  find /opt/drweb.com/ /var/opt/drweb.com/ -type f -exec $md5sum {} \+ > md5sums.txt 2>/dev/null
  #md5 движка
  $md5sum $(drweb-ctl cfshow --value root.coreenginepath 2>&1) > engine.md5 2>&1
fi
showprogress

#лог dmesg
dmesg > logs/dmesg 2>&1
showprogress

#вывод df
df / > df.txt 2>&1
showprogress
df -a >> df.txt 2>&1
showprogress

#вывод ip a/ifconfig -a
{ ip a || ifconfig -a; } > ip.ifconfig.txt 2>&1
showprogress

#вывод ldconfig -p (и аналог для freebsd)
{ ldconfig -p || ldconfig -r; } > ldconfig.txt 2>&1
showprogress

if type drweb-ctl >/dev/null 2>&1; then
#вывод baseinfo -u
  drweb-ctl baseinfo -l > baseinfo.txt 2>&1
#логи drweb (из ini)
  drweb-ctl cfshow --uncut | sed -n 's,^[[:alnum:]]\+\.Log = */,/,p' | \
  while read f; do
    if [ -f "$f" ]; then
      cp "$f" logs/ >> logs/copy.txt 2>&1
    fi
  done
#конфиг drweb.ini, файлы правил или списки из ruleset (@/path/to/file)
  cat /etc/opt/drweb.com/drweb.ini > drweb.ini 2>&1
  drweb-ctl cfshow --uncut | grep -i ruleset > rulesets.txt 2>&1
#hooks
  mkdir -p hooks
  drweb-ctl cfshow --uncut | sed -n 's,.*hook\s*=\s*/,/,ip' | xargs -t -r -I {} cp {} hooks/ > hooks/copy.txt 2>&1
fi
showprogress

#секции [key], [user], и [settings] из drweb32.key если не ec режим
if type drweb-ctl >/dev/null 2>&1 && drweb-ctl license 2>&1 | grep -q 'The license is granted by the protection server'; then
  :
else
  keypath=$(drweb-ctl cfshow --value root.keypath 2>/dev/null)
  if [ -r "$keypath" ]; then
    sed -n '/^\[Key/,/^\s*$/p' "$keypath" > key.txt 2>&1
    sed -n '/^\[User/,/^\s*$/p' "$keypath" >> key.txt 2>&1
    sed -n '/^\[Settings/,/^\s*$/p' "$keypath" >> key.txt 2>&1
  fi
fi
showprogress

#вывод iptables-save
if type iptables-save >/dev/null 2>&1; then
  iptables-save > iptables-save.txt 2>&1
fi
showprogress

#вывод nft export xml
if type nft >/dev/null 2>&1; then
  nft -a list ruleset > nft.txt 2>&1
fi
showprogress

if [ -d /etc/pam.d ]; then
  cp -r /etc/pam.d . >> logs/pam.d.txt 2>&1
fi
showprogress

if [ -r /etc/nsswitch.conf ]; then
  cp /etc/nsswitch.conf . >> logs/nsswitch.conf.txt 2>&1
fi
showprogress

drweb-ctl ap > drweb-ctl.ap.txt 2>> logs/copy.txt
showprogress
drweb-ctl li -d > drweb-ctl.li.txt 2>> logs/copy.txt
showprogress

tar -czf "$d".tgz "$d"
if [ -n "${output}" ]; then
  mv "$d".tgz "${output}"
else
  output="$d".tgz
fi
showprogress

echo "Done. The report saved to ${output}"
