checkRoot() {
    [ -n "$DONT_CHECK_ROOT" ] && return

if [ "`$ID -u`" != "0" ]; then
  echo
  eval_gettext "This script must be executed with root privileges." ; echo
  echo
  exit 1
fi
}

envInit(){
if [ -f /usr/xpg4/bin/awk ]; then
    AWK=/usr/xpg4/bin/awk
else
    AWK=awk
fi

if [ -f /usr/xpg4/bin/sed ]; then
    SED=/usr/xpg4/bin/sed
else
    SED=sed
fi

if [ -f /usr/xpg4/bin/grep ]; then
    GREP=/usr/xpg4/bin/grep
else
    GREP=grep
fi

if [ -f /usr/xpg4/bin/id ]; then
    ID=/usr/xpg4/bin/id
else
    ID=id
fi

if [ -f /usr/xpg4/bin/tr ]; then
    TR=/usr/xpg4/bin/tr
else
    TR=tr
fi

if [ -f /usr/xpg4/bin/tail ]; then
    TAIL=/usr/xpg4/bin/tail
else
    TAIL=tail
fi

if type gpg2 >/dev/null 2>&1; then
    GPG=gpg2
elif type gpg >/dev/null 2>&1; then
    GPG=gpg
fi
}

showyesno() {
if [ "$DRWEB_NON_INTERACTIVE" ] ; then
    return 0
fi
printf_str="$1"
shift
retval=0
while true ; do
    printf "$printf_str (YES/no) " "$*" 2>/dev/null
    read yesno
    case "$yesno" in
        y | yes | Y | Yes | YES | "")
	    retval=0
	    echo "yes"
            break
        ;;
        n | no | N | No | NO)
	    retval=1
	    echo "no"
	    break
        ;;
        *)
            eval_gettext "Please enter yes or no." ; echo
        ;;
    esac
done
return $retval
}

shownoyes() {
if [ "$DRWEB_NON_INTERACTIVE" ] ; then
    return 0
fi
printf_str="$1"
shift
retval=0
while true ; do
    printf "$printf_str (yes/NO) " "$*" 2>/dev/null
    read yesno
    case "$yesno" in
        y | yes | Y | Yes | YES)
	    retval=0
	    echo "yes"
            break
        ;;
        n | no | N | No | NO | "")
	    retval=1
	    echo "no"
	    break
        ;;
        *)
            eval_gettext "Please enter yes or no." ; echo
        ;;
    esac
done
return $retval
}

# $1 - Message type ("info", "warning" or "error")
# $2 - Text (printf string)
show_message() {
    # Process args.
    type=$1
    printf_str=$2
    shift
    shift
    message="`printf \"$printf_str\" \"$@\" 2>/dev/null`"

    # Form window title.
    StringInList 'info warning error' $type || type=info
    case "$type" in
        info) title="`eval_gettext \"Info\"`" ;;
        warning) title="`eval_gettext \"Warning\"`" ;;
        error) title="`eval_gettext \"Error\"`" ;;
    esac

    echo
    echo  '*'
    echo  "* $title: $message" | sed 's/<[^>]\+>//g'
    echo  '*'
}

# $1 - Message type ("info", "warning" or "error")
# $2 - Text (printf string)
gui_message() {
    # Process args.
    type=$1
    printf_str=$2
    shift
    shift
    message="`printf \"$printf_str\" \"$@\" 2>/dev/null`"

    # Form window title.
    StringInList 'info warning error' $type || type=info
    case "$type" in
        info) title="`eval_gettext \"Info\"`" ;;
        warning) title="`eval_gettext \"Warning\"`" ;;
        error) title="`eval_gettext \"Error\"`" ;;
    esac

    # Find some GUI dialog program and form its parameters.
    if type zenity >/dev/null 2>&1; then
        dialog_program=zenity
        type_arg="--$type"
        message_arg="--text '$message' --no-wrap"
    elif type kdialog >/dev/null 2>&1; then
        dialog_program=kdialog
        case "$type" in
            info) type_arg='--msgbox' ;;
            warning) type_arg='--sorry' ;;
            error) type_arg='--error' ;;
        esac
        message_arg="\"$message\""
    else
        dialog_program=
    fi
    title_arg="--title $title"

    # Show the message.
    show_message "$type" "$printf_str" "$@"
    if [ -n "$DISPLAY" ] && [ -n "$dialog_program" ]; then
        eval "$dialog_program" $title_arg $type_arg $message_arg 2>/dev/null
    fi
}

# $1 - Question text
# $2 - Title text
# Returns: 0=yes 1=no 2=no dialog program.
gui_ask() {
    # Process args.
    question_str="$1"
    title_str="$2"

    # Find some GUI dialog program and form its parameters.
    if type zenity >/dev/null 2>&1; then
        dialog_program=zenity
        type_arg="--question"
        question_arg="--text \"$question_str\""
    elif type kdialog >/dev/null 2>&1; then
        dialog_program=kdialog
        type_arg="--yesno"
        question_arg="\"$question_str\""
    else
        dialog_program=
        return 2
    fi
    title_arg="--title '$title_str'"

    # Show the message.
    if [ -n "$dialog_program" ]; then
        eval "$dialog_program $title_arg $type_arg $question_arg"
    else
        false
    fi

    return $?
}

checkNum() {
    echo "$1" | "$GREP" -q '^[0-9][0-9]*$'
    return $?
}

MakeTmpFile() {
    sc_name=`basename $0`
    tm_file=`mktemp /tmp/$sc_name.XXXXXXXXXX`
    if [ ! -f "$tm_file" ]; then
        return 1
    fi
    chmod 644 $tm_file
    echo $tm_file
}

EscapeRE() {
    sed 's/[][\\\/^$.*]/\\&/g'
}

IniPrintSection() {
    ips_sect=`echo "$2" | EscapeRE`
    $SED -n '
        /^[[:space:]]*\['"$ips_sect"'\][[:space:]]*$/ {
            h
    :hold
            n
            /^[[:space:]]*\[/ b end
            H
            $ b end
            b hold
        }
        d
    :end
        g
        p
        q
    ' "$1"
}

# Returns 0 if there is specified key in specified section
# of ini-type config file. Returns 1 otherwise.
#
# $1 - config file
# $2 - section name regexp
# $3 - key name regexp
IniHasKey() {
    IniPrintSection "$1" "$2" | $GREP -q '^[[:space:]]*'"$3"'[[:space:]]*='
}

# Prints a value of some key of specified section of ini-type config file.
#
# $1 - config file
# $2 - section name regexp
# $3 - key name regexp
IniGetValue() {
    IniPrintSection "$1" "$2" | $SED '
        s/^[[:space:]]*'"$3"'[[:space:]]*=[[:space:]]*\(.*\)$/\1/
        t end
        d
    :end
        q
    '
}

# Changes a value of specified parameter in the ini-type config file.
#
# $1 - config file
# $2 - section name regexp
# $3 - key name
# $4 - key name regexp
# $5 - value
# $6 - optional comment
IniSetValue() {
    isv_file="$1"
    isv_sect=`echo "$2" | EscapeRE`
    isv_key="$3"
    isv_keyre="$4"
    isv_val="$5"
    isv_comm="$6"
    isv_tmp=`mktemp drw.ini.XXXXXX`

    if [ ! -f "$isv_tmp" ]; then
        return 1
    fi

    $SED '
        /^[[:space:]]*\['"$isv_sect"'\][[:space:]]*$/ {
    :loop
            n
            /^[[:space:]]*\[/ b insert
            s/^\([[:space:]]*'"$isv_keyre"'[[:space:]]*=[[:space:]]*\).*/\1'"$isv_val"'/
            t endloop
            $ b append
            b loop
    :insert
            i\
'"${isv_comm:+# $isv_comm\\
}\ \ \ ${isv_key} = ${isv_val}"'\

            b endloop
    :append
            a\
'"${isv_comm:+# $isv_comm\\
}\ \ \ ${isv_key} = ${isv_val}"'
            b endloop
    :endloop
        }
    ' "$isv_file" > "$isv_tmp"

    cat "$isv_tmp" > "$isv_file"
    rm -f "$isv_tmp"
}

# GNU/Linux has seq. FreeBSD has jot instead of seq.
# Solaris has neither seq nor jot.
#
# Usage: Seq <from> <to>
Seq() {
    if [ "$1" -gt "$2" ]; then
        return 1
    fi
    i="$1"
    while [ "$i" -le "$2" ]; do
        echo "$i"
        i=`expr $i + 1`
    done
}

# Checks if string is in the list.
#
# $1 - list
# $2 - string
StringInList() {
    if  (echo $1 | grep "^$2[[:space:]]" >/dev/null) ||
        (echo $1 | grep "[[:space:]]$2\$" >/dev/null) ||
        (echo $1 | grep "^$2\$" >/dev/null) ||
        (echo $1 | grep "[[:space:]]$2[[:space:]]" >/dev/null); then
        return 0
    else
        return 1
    fi
}

run_in_terminal() {
    if [ -z "$DISPLAY" -o \( -t 0 -a -t 1 \) ] ; then
        "$@"
        return $?
    fi
    if [ -z "$XDG_CURRENT_DESKTOP" ] ; then
        if [ -n "$GNOME_DESKTOP_SESSION_ID" -o "$DESKTOP_SESSION" = gnome ] ; then
            XDG_CURRENT_DESKTOP=GNOME
        elif [ -n "$MATE_DESKTOP_SESSION_ID" -o "$DESKTOP_SESSION" = mate ] ; then
            XDG_CURRENT_DESKTOP=MATE
        elif [ -n "$DESKTOP_SESSION" ] ; then
            XDG_CURRENT_DESKTOP="$DESKTOP_SESSION"
        fi
    fi
    if [ -n "$XDG_CURRENT_DESKTOP" ] ; then
        OIFS="$IFS"
        IFS=:
        for desktop in $XDG_CURRENT_DESKTOP; do
            IFS="$OIFS"
            case "$desktop" in
            GNOME|Unity|X-Cinnamon)
                if type gnome-terminal >/dev/null 2>&1; then
                    waitfile="`mktemp drwinst.wait.XXXXXX`"
                    gnome-terminal -x sh -c "trap 'rm -f \"$waitfile\"' EXIT HUP INT PIPE TERM; \"\$@\"" '' "$@"
                    while [ -f "$waitfile" ]; do
                        sleep 1
                    done
                    return 0
                fi
                ;;
            MATE)
                if type mate-terminal >/dev/null 2>&1; then
                    mate-terminal --disable-factory -x "$@"
                    return $?
                fi
                ;;
            KDE)
                if type konsole >/dev/null 2>&1; then
                    konsole --nofork -e "$@"
                    return $?
                fi
                ;;
            XFCE)
                if type xfce4-terminal >/dev/null 2>&1; then
                    xfce4-terminal --disable-server -x "$@"
                    return $?
                fi
                ;;
            LXQt)
                if type qterminal >/dev/null 2>&1; then
                    qterminal -e "$@"
                    return $?
                fi
                ;;
            LXDE)
                if type lxterminal >/dev/null 2>&1; then
                    lxterminal -e "$@"
                    return $?
                fi
                ;;
            fly)
                if type fly-term >/dev/null 2>&1; then
                    fly-term -e "$@"
                    return $?
                fi
                ;;
            esac
        done
    fi
    if type urxvt >/dev/null 2>&1; then
        urxvt -e "$@"
    elif type uxterm >/dev/null 2>&1; then
        uxterm -e "$@"
    elif type xterm >/dev/null 2>&1; then
        xterm -e "$@"
    elif type x-terminal-emulator >/dev/null 2>&1; then
        x-terminal-emulator -e "$@"
    else
        "$@"
    fi
    return $?
}

envInit
checkRoot
